/***************************************************************************
   network_icmp.h  -  ICMP network plugin
                             -------------------
    begin                : Wed Jul 03 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: network_icmp.h,v 1.35 2004/05/05 08:01:29 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file network_icmp.h
  * File contains ICMP plugin functions
  * This file is a good example of the plugin API.
  * Use it as an example for your own plugin development.
  */
#ifndef _ICMP_PLUG_H_
#define _ICMP_PLUG_H_

// include the plugin API
#include "plugin.h"

/* Get all the types first */
#include <sys/types.h>
#include <netinet/in_systm.h>

#ifdef MACOSX
#ifndef socklen_t
typedef int socklen_t;
#endif
#endif

#define LINUX 1

#ifndef SOL_IP
#define SOL_IP 0
#endif

#ifndef INADDR_NONE
#define	INADDR_NONE		((in_addr_t) 0xffffffff)
#endif

// include my types
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/time.h>


#define ICMP_MIN 	8

#ifndef ICMP_DEST_UNREACH
#define ICMP_DEST_UNREACH	3
#endif

#ifndef ICMP_PORT_UNREACH
#define ICMP_PORT_UNREACH	3
#endif

/// Maximum receivable packet size
#define MAX_PACKET 1024


/*! \struct recv_t
	\brief Argument passed to the recveiving thread in the icmp plugin
	
	This is for the threaded receiving functionality of the 
	icmp scan plugin.
  */
struct recv_t
{
    //! socket to listen on (must already be open)
    int socket;
    //! number of hosts we pinged
    unsigned int hosts;
    //! set the number of responses
    unsigned int responses;
    //! start time
    unsigned int start;
    //! max wait time (timeout function)
    unsigned int max;
};

/*! \struct icmp_options_s
	\brief Options local to the ICMP scan plugin
*/
typedef struct icmp_options_s
{
	//! Time to live for outgoing packets
    uint8_t ttl;
	//! debug mode (undocumented)
    uint8_t debug;
	//! packet size
    uint32_t size;
} icmp_options_t;

void *plugin_init(void *in_data);

#endif // _ICMP_PLUG_H_
