/***************************************************************************
   rfc793.h  -  TCP ACK Scan plugin
                             -------------------
    begin                : Wed Jul 03 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: rfc793.h,v 1.31 2004/05/05 08:00:32 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file rfc793.h
  * Contains a rfc 793 behavior check (TCP ACK->RST ping)
  */
#ifndef _RFC793_H_
#define _RFC793_H_

// include the plugin API
#include "plugin.h"

#ifdef MACOSX
#ifndef socklen_t
typedef int socklen_t;
#endif
#endif

#define LINUX 1
#define __FAVOR_BSD

#ifndef ICMP_DEST_UNREACH
#define ICMP_DEST_UNREACH       3       /* Destination Unreachable */
#endif

#ifndef ICMP_PORT_UNREACH
#define ICMP_PORT_UNREACH       3       /* Port Unreachable */
#endif

#ifndef SOL_IP
#define SOL_IP 0
#endif

#ifndef INADDR_NONE
#define INADDR_NONE	((in_addr_t) 0xffffffff)
#endif

#define MAX_PACKET 1024

#ifndef __FAVOR_BSD
#define __FAVOR_BSD
#endif
#ifndef __USE_BSD
#define __USE_BSD
#endif
#ifndef __BSD_SOURCE
#define __BSD_SOURCE
#endif

#define int_ntoa(x) inet_ntoa(*((struct in_addr *)&(x)))


// include my types
#include "types.h"
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/time.h>

/** 
 * Checksum stuff 
 */
#define CKSUM_CARRY(x) \
    (x = (x >> 16) + (x & 0xffff), (~(x + (x >> 16)) & 0xffff))

// put your definitions here
#define ETH_SIZE 14
#define IP_SIZE sizeof(struct ip)
#define TCP_SIZE sizeof(struct tcphdr)

// return codes from recv_thread
#define RFC_FILTERED	11
#define RFC_UNFILT		12
#define RFC_UNREACH		-11
#define RFC_NORESP		-12
/*! \struct _fakehead 
	\brief pseudo TCP header for calculating the chksum
	
	Create this header when calculating the TCP checksum
*/
struct _fakehead
{
	//! Source address (network byte order)
    uint32_t saddr;
	//! Destination address (network byte order)
    uint32_t daddr;
	//! always zero
    uint8_t zero;
	//! protocol number
    uint8_t protocol;
	//! total length of packet
    uint16_t tot_len;
};

/*! \struct net_tuple
	\brief leet net tuple
	
	Source and destination addresses (with port numbers) as a net-tuple.
	All values are network byte order
 */
struct net_tuple
{
	//! Source address
    uint32_t src;
	//! Source port
    unsigned short int sport;
	//! Destination address
    uint32_t dst;
	//! Destination port
    unsigned short int dport;
};

/*! \struct rfc_options
	\brief Options local to the rfc793 (ACK scan) plugin
	
	This a structure that is local to rfc793.* and the 
	ack scan plugin.  It contains the ack scan plugin's 
	local options
*/
struct rfc_options
{
	//! Time to live for outgoing packets
    unsigned char ttl;
	//! debug mode (undocumented)
    int debug, 
	//! Broadcast mode
		bcast, 
	//! Source port
		srcPrt;
	//! Program command line to run if host is online
    char *prog;
};

void *plugin_init(void *in_data);

#endif // _RFC793_H_
