/***************************************************************************
   popts.c  -  options parsing functions
                             -------------------
    begin                : Wed Mar 05 2003
    copyright            : (C) 2003 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: popts.c,v 1.13 2004/04/23 04:43:45 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file popts.c
  * File contains options parsing functions
  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "popts.h"
#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

static const char rcsid[] =
    "\100$ autosec/sonar: $Id: popts.c,v 1.13 2004/04/23 04:43:45 red0x Exp $";

extern plugin_list_t network_list[];
extern plugin_list_t input_list[];
extern plugin_list_t output_list[];
extern int num_net, num_inp, num_out;

void
sonar_opt_header()
{
#ifndef DEBUG
    fprintf(stderr, "%s - red0x <red0x@users.sourceforge.net>\nBugs: <%s>\n",
            PACKAGE_STRING, PACKAGE_BUGREPORT);
#else
    fprintf(stderr,
            "%s debug - red0x <red0x@users.sourceforge.net>\nBugs: <%s>" "\n",
            PACKAGE_STRING, PACKAGE_BUGREPORT);
#endif
    fprintf(stderr, "FREE for NON-COMMERCIAL USE\n");

}

void
sonar_opt_usage(int exitcode, char *error, char *addl)
{
    sonar_opt_header();
    fprintf(stderr, " options: [-f|--fork] [-v|--verbose] [-q|--quiet] "
            "[-r|--randomize]\n");
    fprintf(stderr, "\t[--scan_delay=seconds]  [-c|--count cycles] "
            "[-s|--scan_type plugin]\n");
    fprintf(stderr, "\t[-i|--input plugin] [-l|--plugin_list] "
            "[-o|--output plugin]\n");
    fprintf(stderr,
            "\t[-p|--plugin_opt option_string] [--post_delay=mseconds]" "\n");
    fprintf(stderr, "\t[--timeout=mseconds] [-?|--help] [--usage]\n");
    if(error)
        fprintf(stderr, "%s: %s\n", error, addl);
    exit(exitcode);
}

void
sonar_opt_help(int exitcode)
{
    sonar_opt_header();
    fprintf(stderr, "Usage: sonar [options] <scan-type> <target(s)>\n");
    fprintf(stderr, "   -f, --fork                    fork into background "
            "(daemon mode)\n");
    fprintf(stderr, "   -v, --verbose                 increase verbosity\n");
    fprintf(stderr, "   -q, --quiet                   no output, just "
            "return\n");
    fprintf(stderr, "   -r, --randomize               randomize target "
            "order\n");
    fprintf(stderr, "   --scan_delay=seconds          delay between scans "
            "(default: 0)\n");
    fprintf(stderr, "   -c, --count=cycles            number of times to "
            "scan (default: 1)\n");
    fprintf(stderr, "   -s, --scan_type=plugin        scan plugin to "
            "use\n");
    fprintf(stderr, "   -i, --input=plugin            input plugin to "
            "use\n");
    fprintf(stderr, "   -l, --plugin_list             list loaded plugins\n");
    fprintf(stderr, "   -o, --output=plugin           output plugin to "
            "use\n");
    fprintf(stderr, "   -p, --plugin_opt=option       plugin " "option\n");
    fprintf(stderr,
            "   --post_delay=mseconds         delay after each "
            "scan type (default: 0)\n");
    fprintf(stderr,
            "   --timeout=mseconds            timeout for probes "
            "(per host) (default: 5000)\n");
    fprintf(stderr,
            "   --delay=mseconds              time to wait "
            "inbetween hosts (default: 0)\n");
    fprintf(stderr, "   -?, --help                    this cruft.\n");
    fprintf(stderr, "   --usage                       simple usage\n");
    exit(exitcode);
}

void
sonar_opt_pluginlist()
{
    int i;
    plugin_t *plug;

    sonar_opt_header();

    fprintf(stderr, "\n                Currently Loaded Plugins\n");
    fprintf(stderr,
            "Type Cmd             Name                     Description" "\n");
    fprintf(stderr,
            "---- --- ---------------- -------------------------------" "\n");
    for(i = 0; i < num_inp; i++)
    {
        plug = input_list[i].plug;
        fprintf(stderr, "IN   -i%c  %15s  %30s\n", plug->plugin_char,
                plug->shortname, plug->longname);
    }
    for(i = 0; i < num_out; i++)
    {
        plug = output_list[i].plug;
        fprintf(stderr, "OUT  -o%c  %15s  %30s\n", plug->plugin_char,
                plug->shortname, plug->longname);
    }
    for(i = 0; i < num_net; i++)
    {
        plug = network_list[i].plug;
        fprintf(stderr, "NET  -s%c  %15s  %30s\n", plug->plugin_char,
                plug->shortname, plug->longname);
    }
    fprintf(stderr, "\n");
}
