/***************************************************************************
                          sonar.c  -  main sonar file
                             -------------------
    begin                : Mon Apr  8 17:26:46 PDT 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: sonar.c,v 1.44 2004/04/13 03:42:08 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file sonar.c
  * File contains most of the main functions of sonar
  */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <getopt.h>
#include <stdarg.h>
#include <errno.h>
#include <limits.h>

#include "types.h"
#include "sonar.h"
#include "util.h"
#include "plugin.h"
#include "target.h"
#include "net.h"
#include "popts.h"
#include "error.h"
#include "log.h"
#include "mem.h"

#ifdef HAVE_SYSLOG_H
#	include <syslog.h>
#endif

static const char rcsid[] =
    "\100$ autosec/sonar: $Id: sonar.c,v 1.44 2004/04/13 03:42:08 red0x Exp $";

//these are contained in plugin.c
extern plugin_list_t input_list[];
extern plugin_list_t output_list[];
extern plugin_list_t network_list[];
extern uint32_t input_buf[];
extern int num_inp, num_out, num_net;
extern target_t *vectors;

sonar_t mthis;

void
default_context(void *in)
{
    char *temp, *pv;
    sonar_t *mthat;

    assert(in != NULL);

    mthat = (sonar_t *) in;

    memset((void *) mthat, 0, sizeof(sonar_t));
    mthat->ver_major = 0;
    mthat->ver_minor = 0;
    mthat->revision = 0;
    pv = (char *) Malloc(strlen(PACKAGE_VERSION));
    strncpy(pv, PACKAGE_VERSION, strlen(PACKAGE_VERSION));
    temp = strpbrk(pv, ".");
    while(pv != NULL)
    {
        if(temp)
            *temp++ = 0x00;
        if(mthat->ver_major == 0)
            mthat->ver_major = atoi(pv);
        else if(mthat->ver_minor == 0)
            mthat->ver_minor = atoi(pv);
        else if(mthat->revision == 0)
            mthat->revision = atoi(pv);
        else
            pv = NULL;
        pv = temp;
        if(!pv)
            break;
        temp = strpbrk(pv, ".");
    }
    free(pv);
    // plugin API number
    mthat->api = API_VER;
    mthat->memput = &bput;
    mthat->message = &sonar_msg;
    mthat->resolve = &Host_serv;
    mthat->output = &sonar_rawoutput;
    mthat->usage = &sonar_usage;
    mthat->getprivs = &sonar_getprivs;
    mthat->dropprivs = &sonar_dropprivs;
    mthat->find_target = &sonar_find_target;
    // no targets yet ;)
    mthat->vectors = NULL;
    mthat->num_targets = 0;
    mthat->num_ports = 0;
}


/**
  * @fn main(int argc, char *argv[])
  * Main Method
  * @param argc Number of arguments
  * @param argv Arguments
  * @return SONAR_OK on success
  */
int
main(int argc, char *argv[])
{

    int cur = 0, opt = 0, ret, opt_ind = 1;
    int plugin_selected = 0, i;
    int verbosity = 0;
    target_t *first_target;
    struct timeval ts, te;
    float dif = 0;
    static char *plugin;
    int n;

    struct option optionsTable[] = {
        GETOPT_OPTIONS {"help", 0, NULL, '?'},
        {"usage", 0, NULL, 'U'},
        {0, 0, 0, 0}
    };

    // setup mthis (sonar_t)
    default_context(&mthis);

    mthis.my_name = argv[0];
    clear_opts(&(mthis.opts));

    // these will be fatal errors, as plugins become the 
    // core of sonar
    log_write(LOG_DBUG, "plugin_start(%p);\n", &mthis);

    if(plugin_start(&mthis) != PLUGIN_OK)
        sonar_opt_usage(-1, "Fatal", "Could not initialize plugin API\n");

    log_write(LOG_DBUG, "plugin_loadall(%p);\n", &mthis);

    if(plugin_loadall(&mthis) != PLUGIN_OK)
        sonar_opt_usage(-1, "Fatal", "Could not load any plugins\n");

    if(argc < 2)
    {
        sonar_opt_usage(0, "error", "no arguments\n");
    }

    while((opt = getopt_long(argc, argv, "v::qrfld:c:s:i:o:p:w:t:h:be:?",
                             optionsTable, &opt_ind)) >= 0)
    {
        // core options, handled first
        switch (opt)
        {
            /* select a plugin */
        case 's':
            plugin = (char *) optarg;
            if(!plugin)
                sonar_opt_usage(-1, "fatal", "no plugin selected");
            for(i = 0; i < num_net; i++)
            {
                char *temp;
                int len, tlen = strlen(plugin);
                int len2 = strlen(network_list[i].plug->shortname);
                len = (tlen < len2) ? tlen : len2;
                if((strncmp(plugin, network_list[i].plug->shortname, len)
                    == 0) || (char) *plugin ==
                   network_list[i].plug->plugin_char)
                {
                    plugin_selected++;
                    log_write(LOG_DBUG,
                              "found network plugin \'%c\': %s\n",
                              network_list[i].
                              plug->plugin_char,
                              network_list[i].plug->shortname);
                    network_list[i].plug->selected = true;
                    temp = strstr(plugin, ",");
                    if(!temp)
                    {
                        if(network_list[i].plug->plugin_open(NULL, 0) !=
                           PLUGIN_OK)
                        {
                            log_write(LOG_DBUG, "could not plugin_open"
                                      "(NULL, 0);\n");
                            network_list[i].plug->ready = false;
                        }
                    }
                    else if(*temp++)
                    {
                        if(network_list[i].plug->plugin_open(temp,
                                                             strlen(temp)) !=
                           PLUGIN_OK)
                        {
                            log_write(LOG_DBUG, "could not plugin_open"
                                      "(\'%s\', %d);\n", temp, strlen(temp));
                            network_list[i].plug->ready = false;
                            /* leave it as selected, but not ready, so it
                             * will print the usage */
                        }
                    }
                }
            }
            break;
        case 'i':
            plugin = (char *) optarg;
            if(!plugin)
                sonar_opt_usage(-1, "fatal", "no plugin selected");

            for(i = 0; i < num_inp; i++)
            {
                char *temp;
                int n;
                int len, tlen = strlen(plugin);
                int len2 = strlen(input_list[i].plug->shortname);
                len = (tlen < len2) ? tlen : len2;
                if((strncmp(plugin, input_list[i].plug->shortname, len)
                    == 0) || (char) *plugin ==
                   input_list[i].plug->plugin_char)
                {
                    log_write(LOG_DBUG, "found input plugin \'%c\': %s"
                              "\n",
                              input_list[i].plug->
                              plugin_char, input_list[i].plug->shortname);
                    plugin_selected++;
                    input_list[i].plug->selected = true;
                    temp = strstr(plugin, ",");
                    if(!temp)
                    {
                        if((n = input_list[i].plug->plugin_open(NULL, 0))
                           <= 0)
                        {
                            log_write(LOG_DBUG, "could not plugin_open"
                                      "(NULL, 0);\n");
                            input_list[i].plug->ready = false;
                        }
                        input_buf[i] = n;
                    }
                    else if(*temp++)
                    {
                        if((n = input_list[i].plug->plugin_open(temp,
                                                                strlen(temp)))
                           <= 0)
                        {
                            log_write(LOG_DBUG, "could not plugin_open"
                                      "(\'%s\', %d);\n", temp, strlen(temp));
                            input_list[i].plug->ready = false;
                            /* leave it as selected, but not ready, so it
                             * will print the usage */
                        }
                        input_buf[i] = n;
                    }
                }
            }
            break;
        case 'o':
            plugin = (char *) optarg;
            if(!plugin)
                sonar_opt_usage(-1, "fatal", "no plugin selected");
            for(i = 0; i < num_out; i++)
            {
                char *temp;
                int len, tlen = strlen(plugin);
                int len2 = strlen(output_list[i].plug->shortname);
                len = (tlen < len2) ? tlen : len2;
                if((strncmp(plugin, output_list[i].plug->shortname, len)
                    == 0) || (char) *plugin ==
                   output_list[i].plug->plugin_char)
                {
                    plugin_selected++;
                    log_write(LOG_DBUG,
                              "found output plugin \'%c\': %s\n",
                              output_list[i].
                              plug->plugin_char,
                              output_list[i].plug->shortname);
                    output_list[i].plug->selected = true;
                    temp = strstr(plugin, ",");
                    if(!temp)
                    {
                        if(output_list[i].plug->plugin_open(NULL, 0)
                           != PLUGIN_OK)
                        {
                            log_write(LOG_DBUG, "could not "
                                      "plugin_open(NULL, 0);\n");
                            output_list[i].plug->ready = false;
                        }
                    }
                    else if(*temp++)
                    {
                        if(output_list[i].plug->plugin_open(temp,
                                                            strlen(temp)) !=
                           PLUGIN_OK)
                        {
                            log_write(LOG_DBUG, "could not "
                                      "plugin_open(\'%s\', %d);\n",
                                      temp, strlen(temp));
                            output_list[i].plug->ready = false;
                            /* leave it as selected, but not ready, so it
                             * will print the usage */
                        }
                    }
                }
            }
            break;
        case 'v':
            if(optarg != NULL)
            {
                /* atoi does not set errno */
                verbosity = strtol(optarg, (char **) NULL, 10);
                if(errno == EINVAL)
                    verbosity = 0;
                if(errno == LONG_MIN)
                    verbosity = -1;
                if(errno == LONG_MAX)
                    verbosity = MAX_VERBOSE;
                if(errno == ERANGE)
                    verbosity = 1;
                /* red0x: added sanity check */
                verbosity = sanity_check(verbosity, MAX_VERBOSE, MORE);
                mthis.opts.iVerbose = sanity_check(verbosity, 0, LESS);
            }
            else
                mthis.opts.iVerbose = 1;
            log_write(LOG_DBUG, "verbosity = %d;\n", mthis.opts.iVerbose);
            break;
        case 'l':
            sonar_opt_pluginlist();
            exit(0);
            break;
        case 'q':
            mthis.opts.iVerbose = -1;
            break;
        case 'd':
            mthis.opts.iPostDelay = sanity_check(atoi(optarg), 0, LESS);
            break;
        case 'c':
            mthis.opts.iCount = sanity_check(atoi(optarg), 0, LESS);
            break;
        case 'w':
            mthis.opts.iPostDelay = sanity_check(atoi(optarg), 0, LESS);
            break;
        case 't':
            mthis.opts.iTimeout = sanity_check(atoi(optarg), 0, LESS);
            if(mthis.opts.iTimeout < 500)
                log_write(LOG_CONSOLE, "low timeout chosen, results"
                          "could be unreliable\n");
            break;
        case 'h':
            mthis.opts.iHostDelay = sanity_check(atoi(optarg), 0, LESS);
            break;
        case 'r':
            mthis.opts.bRandom = 1;
            break;
        case 'f':
            daemon_proc(argv[0], LOG_DAEMON);
            break;
        case '?':
            sonar_opt_help(0);
            break;
        case 'U':
            sonar_opt_usage(0, NULL, NULL);
            break;
        default:
            sonar_opt_usage(-1, "Bad Argument", "unknown argument " "passed");
            break;
        }
    }

    if(plugin_selected == 0)
    {
        sonar_opt_usage(-1, "fatal", "no plugin(s) selected");
    }

    /* no hostnames on the command line */
    log_write(LOG_DBUG, "optind: %d, argc: %d\n", optind, argc);
    if(optind >= argc)
    {
        int i, iplug = 0;
        char *hostname;

        log_write(LOG_DBUG,
                  "no hosts, checking for input plugs, then "
                  "printing usage\n");

        for(i = 0; i < num_inp; i++)
        {
            /* see if we have input plugins selected */
            if(input_list[i].plug->selected == true)
            {
                iplug++;
                assert(input_list[i].plug->plugin_open != NULL);
                assert(input_list[i].plug->plugin_usage != NULL);
                /* not ready */
                if(!input_list[i].plug->ready)
                {
                    input_list[i].plug->plugin_usage();
                    continue;
                }
                /* input_buf[i] has the buffer size */
                hostname = (char *) Malloc(sizeof(char) * input_buf[i]);
                do
                {
                    memset(hostname, 0, input_buf[i]);
                    if(input_list[i].plug->plugin_next(hostname) != PLUGIN_OK)
                        break;
                    sonar_add_target(&mthis, hostname);
                }
                while(1);
                free(hostname);
                break;
            }
        }
        if(iplug == 0)
        {
            for(i = 0; i < num_out; i++)
            {
                if(output_list[i].plug->selected == true)
                {
                    assert(output_list[i].plug->plugin_usage != NULL);
                    output_list[i].plug->plugin_usage();
                }
            }
            for(i = 0; i < num_net; i++)
            {
                if(network_list[i].plug->selected == true)
                {
                    assert(network_list[i].plug->plugin_usage != NULL);
                    network_list[i].plug->plugin_usage();
                }
            }
            exit(-1);
        }
    }
    sonar_commandline(argv, argc, &(mthis.opts.sCommandLine));
    while(optind < argc)
    {
        plugin = (char *) argv[optind++];
        // pass off anything left as targets

        log_write(LOG_DBUG, "sonar_add_target(%s);\n", plugin);
        sonar_add_target(&mthis, plugin);

    }

#ifdef DEBUG
    sonar_debug_targets(mthis.vectors);
#endif
    if(mthis.opts.bRandom)
        /* randomize the targets */
        mthis.vectors = randomize(mthis.vectors);
    else                        /* randomize takes care of looping, but if we 
                                 * didn't randomize, loop it */
    {
        first_target = mthis.vectors;   /* all targets added, loop it */
        assert(mthis.vectors != NULL);
        while(mthis.vectors->next != NULL)
        {
            mthis.vectors = mthis.vectors->next;
        }
        mthis.vectors->next = first_target;
        mthis.vectors = first_target;
    }

    signal(SIGTERM, sig_int);
    signal(SIGINT, sig_int);
    signal(SIGHUP, sig_int);
#ifdef	DEBUG
    signal(SIGSEGV, sig_int);
#endif

    /* Status report here */
    if(mthis.opts.iVerbose >= 0)
        sonar_status_report(mthis);

    if(gettimeofday(&ts, NULL) != 0)
        err_ret("gettimeofday failed");

    for(i = 0; i < num_net; i++)
    {
        assert(network_list[i].plug != NULL);
        if(network_list[i].plug->selected)
        {
            if(!network_list[i].plug->ready)
                network_list[i].plug->plugin_open(NULL, 0);
        }
    }
    while(mthis.opts.iCount == -1 || cur < mthis.opts.iCount)
    {
        int err;
        if((err = sonar_sweep(mthis.vectors)) == PLUGIN_ERROR)
        {
            sonar_msg("Sweep ended with a plugin error\n");
            return SONAR_ERROR;
        }
        if(err == PLUGIN_OK)
        {
            /* the meaning of this has changed slightly, */
            /* it just means everything went alright, no errors */
        }
        cur++;

        /* XXX: There's a bug here- signals aren't caught while it's sleeping 
         */
        /* i think usleep or nanosleep will fix this - rd */
        if((mthis.opts.iCount > 1 || mthis.opts.iCount == -1)
           && (cur != mthis.opts.iCount))
            usleep(mthis.opts.iPostDelay * 1000);
    }
    for(n = 0; n < num_out; n++)
    {
        if(output_list[n].plug->selected && output_list[n].plug->ready)
        {
            /* let each plugin parse its own results */
            ret = output_list[n].plug->plugin_results(mthis.
                                                      vectors,
                                                      mthis.num_targets);
            if(ret != PLUGIN_OK)
                log_write(LOG_CONSOLE | LOG_SLOG,
                          "output plugin \'%s\' had trouble "
                          "writing output\n", output_list[i].plug->shortname);
        }
    }
    if(gettimeofday(&te, NULL) != 0)
        err_ret("gettimeofday failed");
    tvsub(&te, &ts);
    dif = te.tv_sec + te.tv_usec / 1000000.0;
    /* XXX: i took LOG_SLOG out because it would hang in daemon mode */
    log_write(LOG_CONSOLE | LOG_PLUGINS, "Scan took %.2f seconds\n", dif);

    if(plugin_killall() != PLUGIN_OK)
        err_msg("Could not unload plugins\n");
    if(plugin_end() != PLUGIN_OK)
        err_msg("could not destroy plugins\n");

    return SONAR_OK;
}

/** @fn sonar_getprivs(void)
  * Get root priveleges when needed
  * @return uid of user before we got root
  */
int
sonar_getprivs(void)
{
    uid_t uid;
    uid = getuid();
    log_write(LOG_DBUG, "  getuid() = %d;\n", uid);
    setreuid(-1, 0);
    log_write(LOG_DBUG, "  setreuid(-1,0);\n");

    if(geteuid())
        sonar_usage("must be root!");
    return uid;
}

/** @fn sonar_dropprivs(uid_t uid)
  * Drop root priveleges when no longer needed
  * Drops down to specified euid
  * @param uid UID_T of user to drop to
  * @return SONAR_OK 
  */
int
sonar_dropprivs(uid_t uid)
{
    if(uid != geteuid())        /* no need to drop priveleges, its not setuid 
                                 */
    {
        log_write(LOG_DBUG, "seteuid(%d);\n", uid);
        if(seteuid(uid) != 0)
        {
            sonar_msg("could not drop priveleges, halting\n");
            exit(-1);
        }
    }
    return SONAR_OK;
}

/** @fn sonar_rawoutput(char *msg,...)
  * Output formatted text given by msg and any other arguments.
  * Used in the Plugin API
  * @param msg Format string to output (see printf)
  * @return SONAR_OK 
  * @todo remove all dependency on this function.  Instead, switch to log_write
  */
int
sonar_rawoutput(char *msg, ...)
{
    va_list ap;
    if(mthis.opts.bFork)
        return SONAR_OK;

    va_start(ap, msg);
    /* XXX: the sCon struct has been removed, so oFile doesn't exist any
     * more. vfprintf(mthis.sCon.oFile, msg, ap); */
    /* TODO: when updating plugin to have oFiles from local params, change
     * the following places with stdout */
    vfprintf(stdout, msg, ap);
    va_end(ap);
    fflush(stdout);
    return SONAR_OK;
}

/** @fn sonar_status_report(sonar_t mthis)
  * Output a status report before each scan
  * @param mthis sonar_t structure
  */
void
sonar_status_report(sonar_t mthis)
{
    char buf[128];
    struct tm *tm;
    time_t now = time(NULL);
    if(!(tm = localtime(&now)))
        err_sys("Unable to get current localtime()#!#");

    /* ISO 8601 date/time: http://www.cl.cam.ac.uk/~mgk25/iso-time.html */
    if(strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M %Z", tm) <= 0)
        err_quit("Could not get date\n");
    if(!mthis.opts.bFork && mthis.opts.iVerbose >= 0)
    {
        log_write(LOG_CONSOLE | LOG_SLOG | LOG_PLUGINS,
                  "starting sonar %s sweep at %s\n", VERSION, buf);
        if(mthis.opts.iVerbose > 0)
            log_write(LOG_CONSOLE | LOG_DBUG | LOG_PLUGINS,
                      "%s\n", mthis.opts.sCommandLine);
        log_write(LOG_CONSOLE, "");
        plugin_status_hook();
        log_write(LOG_CONSOLE, "\n");
    }
}

/** @fn sonar_sweep(target_t *targets)
  * @param targets Pointer to list of targets
  * Run one pass of a sonar sweep
  * @return SONAR_ERROR if sweep failed
  * @return SONAR_OK if sweep worked
  * @todo Write some kind of error reporting/handling code, and have all the
  * plugins return real error values
  */
int
sonar_sweep(target_t * targets)
{
    // pick which network plugin to use, by getting the
    // ready one(s) and running their sweep functions.
    // these functions wont return until someone presses control-C, 
    // unless otherwise noted, at which point, program will return
    int status = PLUGIN_OK, plug;
    for(plug = 0; plug < num_net; plug++)
    {
        if(network_list[plug].plug == NULL)
        {
            // broken plugin
            err_msg("BROKEN PLUGIN DETECTED: id: %d -> %s",
                    plug, network_list[plug].plug->shortname);
            continue;
        }
        {
            assert(network_list[plug].plug != NULL);
            assert(network_list[plug].plug->plugin_sweep != NULL);
        }

        // only run this sweep if the plugin is ready
        if(network_list[plug].plug->ready == true
           && network_list[plug].plug->selected == true)
        {
            status = network_list[plug].plug->plugin_sweep(targets);
#ifdef DEBUG
            err_msg("network_list[%d].plug->plugin_sweep(%p); => %d", plug,
                    targets, status);
#endif
            /* XXX: add some sort of error reporting here */
        }
        if(status != PLUGIN_OK)
            return status;
    }
    return SONAR_OK;
}
