/***************************************************************************
                          sonar.h  -  description
                             -------------------
    begin                : Tue May 15 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SONAR_H_
#define SONAR_H_

#include "plugin.h"
/// Error
#define SONAR_ERROR -1
/// Ok
#define SONAR_OK    1
// Trying to avoid magic numbers
#define MAX_VERBOSE 5

#define GETOPT_OPTIONS {"fork", 0, NULL, 'f'}, \
	{"verbose", 0, NULL, 'v'}, \
	{"quiet", 0, NULL, 'q'}, \
	{"randomize", 0, NULL, 'r'}, \
	{"scan_delay", 1, NULL, 'd'}, \
	{"count", 1, NULL, 'c'}, \
	{"scan_type", 1, NULL, 's'}, \
	{"input", 1, NULL, 'i'}, \
	{"output", 1, NULL, 'o'}, \
	{"plugin_opt", 1, NULL, 'p'}, \
	{"post_delay", 1, NULL, 'w'}, \
	{"timeout", 1, NULL, 't'}, \
	{"delay", 1, NULL, 'h'}, \
	{"debug", 0, NULL, 'b'}, \
    {"plugin_list", 0, NULL, 'l'}, \

int h_errno;

int sonar_getprivs(void);
int sonar_dropprivs(uid_t);
void sonar_status_report(sonar_t mthis);
int sonar_rawoutput(char *msg, ...);
int sonar_sweep(target_t * targets);
void default_context(void *in);

/*! \mainpage sonar documentation and manual
 *
 * \section intro Introduction
 *
 * Sonar is used for network reconnaisance. Since release 1.0,
 * it has been plugin driven, with most of the control coming
 * from the plugins.  By default, version 1.2.2 comes with the
 * default ICMP sweep plugin, as well as the ACK plugin.  
 * This plugin can handle multiple
 * host names, so porting any scripts from sonar pre 1.0 to 1.0
 * should not be too much of a problem.  \n\n
 * Read INSTALL for generic instructions on compiling and installing.
 * This product is under the GNU GPL, which is available for reading
 * in COPYING.  Contributors, feel free to send me patches.  If I
 * include them in a release, you get your name and email (if you want)
 * in AUTHORS.  \n\n
 * This software comes with ABSOLUTELY NO WARRENTY, IMPLIED OR EXPRESSED!
 *
 * \section description Description
 *
 *  This software is used for the automatic probing of internet hosts
 *  at a timed interval (reconnaisance), checking host connectivity,  
 *  especially ICMP support (information gathering), automated running
 *  of third party tools when said host is online (automated testing).
 *  Using sonar is solely the user's or users' responsibility. In other words, 
 *  the author cannot be held responsible for your actions in any way.
 *
 *
 * \section cppapi Creating Plugins and the Plugin API
 * 
 *  See the \link pluginapi Plugin API Manual \endlink. The plugin API is 
 *  currently version 6.
 *
 *
 * \section install Installation
 *
 * \subsection step1 Step 1: Getting sources
 *  Sources are available at  http://autosec.sourceforge.net .  
 *  For those bleeding edgers out there,
 *  current work is available via anonymous CVS with the tag "HEAD". 
 *  Instructions are at http://sourceforge.net/cvs/?group_id=51027 .
 * \subsection step2 Step 2: Extraction and setup
 *  \c tar \c xzvf \c sonar-P-Q.tar.gz  \n
 *  \c cd \c sonar-P-Q  \n
 *  \c ./configure  \n\n
 *  Then edit config.h if you need to.
 * \subsection step3 Step 3: Compiling and Installing
 *  \c gmake  \n
 *  \c su \c root  \n
 *  \c gmake \c install  \n\n
 *  This will \b NOT install sonar SETUID root.  If you want sonar to be setuid root,
 *  type this:  \n\n
 *  \c chmod \c u+s \c /usr/local/bin/sonar \n\n
 * To change the install location, try this: \n
 * \c ./configure \c --prefix=/usr
 *  
 * \subsection help Further Help
 * Further help is available in the INSTALL and README files.
 *
 * \section probs Install Problems and Bugs
 *  If you are running a *nix platform, feel free to submit a 
 *  bug report about the compile process, or any other part of sonar.  
 *  Be sure to include \b ALL error messages during
 *  compile or crash, and feel free to attach core dumps, they help.  
 *  These reports should be submitted to 
 *  http://autosec.sourceforge.net/helpDesk/index.htm \n\n
 *  Known bugs are displayed at that site. 
 *
 * \section examples Syntax Examples
 *  \c sonar \c -sI,ttl=250 \c www.google.com  \n
 *    Sends pings to www.google.com with a TTL of 250 \n\n
 * \c sonar \c --scan_delay=5 \c -sI \c www.yahoo.com  \n
 *    Pings yahoo.com with a 5 second delay  \n\n
 * \c sonar \c -sI,payload=/etc/passwd,size=300 \c genbukan.no-ip.com  \n
 *    Sends the first 300 bytes of your passwd file to genbukan security \n
 *     (why would you want to do that?)  \n\n
 * \c sonar \c -f \c-oL,scan.log \c -c \c 5 \c --scan_delay=300 
 * \c -sA,ttl=220,prog="echo google's up" \c www.google.com:80  \n
 *    Puts sonar into the background (daemon mode), output to scan.log, send \n
 *    5 ACK probes at most, once every 5 minutes, set the TTL to 220, and send\n
 *    the pings to www.google.com, port 80. If the host responds, it \n
 *    prints "google's up" on the command line.  \n\n
 * \c sonar  \n
 * \c sonar \c -l \n
 *    Gets the usage of sonar and lists all your plugins \n\n
 * \c sonar \c -sA \c www.google.com \n
 *    Does an ACK probe of google.com on all ports listed in /etc/services! \n
 *  Instead, try this:
 * \c sonar \c -sA \c www.google.com:80 \n
 *
 * \section contact Contact Information
 *   \c mail: \c red0x \c <red0x@users.sourceforge.net>  \n
 *   \c irc: \c irc.freenode.org \c - \c #autosec \c - \c red0x \n
 *   \c web: \c http://autosec.sourceforge.net/helpDesk/index.htm
 */



#endif
