/***************************************************************************
                          util.cpp  -  description
                             -------------------
    begin                : Wed Apr 17 2002
    copyright            : (C) 2002 by red0x
    email                : red0x@users.sourceforge.net
    rcsid                : $Id: util.c,v 1.29 2004/04/13 03:42:08 red0x Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/** @file util.c
  * File contains some helper functions
  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "util.h"
#include "types.h"
#include "plugin.h"
#include "popts.h"
#include "error.h"
#include "log.h"
#include "mem.h"
#include <pthread.h>
#include <assert.h>
#include <syslog.h>
#include <sys/stat.h>


extern int sonar_rawoutput(char *msg, ...);
extern int sonar_sweep(void);
extern sonar_t mthis;

static const char rcsid[] =
    "\100$ autosec/sonar: $Id: util.c,v 1.29 2004/04/13 03:42:08 red0x Exp $";

/**
  * @fn sonar_usage(const char *err)
  * Print the usage with and error message
  * @param err Error to report
  */
void
sonar_usage(const char *err)
{
    sonar_opt_usage(-1, "error", (char *) err);
}

/**
  * @fn bput(char ** dest, const char * src, const int len)
  * Copy memory from one place to another
  * @param dest Double pointer to destination
  * @param src Pointer the source
  * @param len Length of source
  * @todo Deprecate using this
  */
void
bput(char **dest, const char *src, const int len)
{
    *dest = (char *) Malloc(len + 2);
    memset(*dest, 0, len + 2);
    strncpy(*dest, src, len);
//  return dest;
}

/**
  * @fn sonar_commandline(char *argv[], int argc, char **out)
  * Get the commandline and store it in *out
  * @param argv argv from main
  * @param argc argc from main
  * @param out Double pointer to destination
  */
void
sonar_commandline(char *argv[], int argc, char **out)
{
    int i, len = 0;

    for(i = 0; i < argc; i++)
        len += strlen(argv[i]) + 1;

    *out = (char *) Malloc(len + 2);
    memset(*out, 0, len + 2);

    for(i = 0; i < argc; i++)
    {
        strncat(*out, argv[i], strlen(argv[i]));
        strcat(*out, " ");
    }
}

/**
  * @fn sonar_msg(char *msg, ...)
  * Print a message to plugins and syslog if in daemon mode, or to the plugins
  * and console otherwise
  * @param msg Format specifier like printf
  */
void
sonar_msg(char *msg, ...)
{
    va_list ap;
    char *buf;


    va_start(ap, msg);
    buf = (char *) Malloc(128);
    memset(buf, 0, 128);
    va_start(ap, msg);
    vsnprintf(buf, 127, msg, ap);

    if(mthis.opts.bFork)
        log_write(LOG_PLUGINS | LOG_SLOG, buf);
    else
        log_write(LOG_PLUGINS | LOG_CONSOLE, buf);
    free(buf);
    va_end(ap);
}

RETSIGTYPE
sig_int(int i)
{
    switch (i)
    {
    case SIGINT:
        err_msg("caught SIGINT, cleaning up..\n");
        break;
    case SIGTERM:
        err_msg("caught SIGTERM, cleaning up..\n");
        break;
    case SIGHUP:
        err_msg("caught SIGHUP, restarting..\n");
        sonar_sweep();
        break;
    case SIGSEGV:
        err_msg("oops! sonar segmentation fault.\n");
        /* plugin_killall(); */
        abort();
        break;
    }
    /* plugin_killall(); */

    exit(0);
}

struct stat *
get_stats(const char *file)
{
    struct stat *buf;
    char *path = NULL, *token, *str, cwd[PATH_MAX];

    assert(file != NULL);
    buf = (struct stat *) Malloc(sizeof(struct stat));

    log_write(LOG_DBUG, "   get_stats: checking for %s\n", file);

    // if we have the full path
    if(stat(file, buf) == 0)
        return buf;
    log_write(LOG_DBUG, "   get_stats: not found, checking in $PATH\n");

    path = getenv("PATH");
    // check in the paths
    if(path)
    {
        log_write(LOG_DBUG, "   get_stats: $PATH=\"%s\"\n", path);
        token = strtok(path, ":");
        while(token != NULL)
        {
            // check this location for file
            int tok = strlen(token);
            int pth = strlen(file);
            int len = tok + pth + 2;

            str = (char *) Malloc(len);
            memset(str, 0, len);
            // build the path name
            strncpy(str, token, tok);
            strncat(str, "/", 1);
            strncat(str, file, pth);
            log_write(LOG_DBUG, "   get_stats: trying %s\n", str);

            // try to stat
            if(stat(str, buf) == 0)
                return buf;
            // not found, clear the mem
            memset(str, 0, len);
            // free the mem
            free(str);
            // next token
            token = strtok(NULL, ":");
        }
    }
    // not found in path? check current working dir
    memset(cwd, 0, PATH_MAX);
    getcwd(cwd, PATH_MAX - 1);
    log_write(LOG_DBUG, "   get_stats: trying cwd\n");
    // we can get the cwd
    if(cwd)
    {
        // check here for file
        int pth = strlen(file);
        int lcw = strlen(cwd);
        int len = pth + lcw + 2;

        str = (char *) Malloc(len);
        memset(str, 0, len);
        // build the path name
        strncpy(str, cwd, lcw);
        strncat(str, "/", 1);
        strncat(str, file, pth);
        log_write(LOG_DBUG, "   get_stats: trying %s\n", str);
        // try to stat
        if(stat(str, buf) == 0)
            return buf;
        // not found, clear mem
        memset(str, 0, len);
        // free
        free(str);
    }
    log_write(LOG_DBUG, "   could not find/stat %s\n", file);
    return NULL;
}

RETSIGTYPE
sig_chld(int sig)
{
    pid_t pid;
    int stat;

    pid = wait(&stat);
#if RETSIGTYPE == void
    return;
#else
    return (0);
#endif
}

/**
  * @fn daemon_proc(const char *pname, int facility)
  * Set things up for a daemon process 
  * @param pname Program name to report in syslog
  * @param facility Facility under which to open syslog
  * @todo This function closes file descriptors associated with output plugins,
  * We need to fix that
  */
void
daemon_proc(const char *pname, int facility)
{
    int i;
    pid_t pid;

    assert(pname != NULL);
    log_write(LOG_DBUG, "  Going into daemon mode\n");

    if((pid = fork()) != 0)
        exit(0);                /* parent terminates */

    /* 1st child continues */
    setsid();                   /* become session leader */

    signal(SIGHUP, SIG_IGN);
    signal(SIGCHLD, sig_chld);
    if((pid = fork()) != 0)
        exit(0);                /* 1st child terminates */

    /* 2nd child continues */
    mthis.opts.bFork = 1;       /* for our err_XXX() functions */

    chdir("/");                 /* change working directory */

    umask(0);                   /* clear our file mode creation mask */

    /* XXX: changed MAXFD to 3 */
    for(i = 1; i < MAXFD; i++)
        close(i);

    openlog(pname, LOG_PID, facility);
}

/** 
  * @fn escape_shell(char **myfunc)
  * Escape shell meta-characters from strign myfunc
  * @param myfunc String with possible meta-characters to be escaped
  * @return escaped string in the pointer
  */
void
escape_shell(char **myfunc)
{
    char *func;
    int len;
    int i = 0;
    if(myfunc == NULL)
        return;
    if(*myfunc == NULL)
        return;

    func = *myfunc;
    len = strlen(func);
    for(i = 0; i < len; i++)
    {
        if(func[i] == '>' || func[i] == '<' ||
           func[i] == ';' || func[i] == '(' ||
           func[i] == '`' || func[i] == '!' ||
           func[i] == '|' || func[i] == '&' ||
           func[i] == '*' || func[i] == '?' ||
           func[i] == '$' || func[i] == ':' ||
           func[i] == '=' || func[i] == '#' ||
           func[i] == '!' || func[i] == '%' ||
           func[i] == ')' || func[i] == '[' ||
           func[i] == ']' || func[i] == '{' || func[i] == '}' ||
           /* somone trying to pass shellcode w/nop's? */
           func[i] == (char) 0x90)
            // we want to get rid of these kinds of characters
            // so, move the string forward
        {

            char *tmp;
            if(i > 0)
                if(func[i - 1] == '\\')
                    continue;
            log_write(LOG_DBUG,
                      "   :Found Meta-char(\\%c): escaping\n", func[i]);

            // now we will escape the offending char
            // make temp storage, one larger than offending string
            tmp = (char *) Malloc(++len);
            // clear it
            memset(tmp, 0, len);
            // move the first half of the string to the temp
            memmove(tmp, func, i);
            // inject the escape char
            tmp[i] = '\\';
            // move second half of string to the temp
            memmove(&tmp[i + 1], &func[i], len - (i + 1));
            // swap temp and active variables
            realloc(func, len + 1);
            memset(func, 0, len + 1);
            memmove(func, tmp, len);
            free(tmp);

        }
    }
}

/**
  * @fn tvsub(register struct timeval *out, register struct timeval *in)
  * Subtract one struct timeval from another
  * @param out Pointer to destination (will be filled with out - in)
  * @param in Pointer to src
*/
void
tvsub(register struct timeval *out, register struct timeval *in)
{
    if((out->tv_usec -= in->tv_usec) < 0)
    {
        out->tv_sec--;
        out->tv_usec += 1000000;
    }
    out->tv_sec -= in->tv_sec;
}

/**
  * @fn randomize(target_t * in)
  * Randomize the list of targets
  * @param in Final, filled in linked list of results, before you make it
  * circular
  * @return A copy of the linked list in random order
  */
target_t *
randomize(target_t * in)
{
    target_t *hosts, *hosts2;
    int i;
    unsigned int rand_int, num_hosts = mthis.num_targets;

    assert(in != NULL);

    /* if we dont have enough to change or we dont want it random */
    if(!in->next || mthis.num_targets == 1 || !mthis.opts.bRandom)
        return in;

    /* get me a couple of arrays */
    hosts = (target_t *) Malloc(sizeof(target_t) * num_hosts);
    hosts2 = (target_t *) Malloc(sizeof(target_t) * num_hosts);
    memset(hosts, 0, sizeof(target_t) * num_hosts);
    memset(hosts2, 0, sizeof(target_t) * num_hosts);

    if(in->addrinfo == 0)
        in = in->next;

    /* get the source array */
    for(i = 0; i < num_hosts; i++)
    {
        if(in != NULL)
        {
            hosts[i].entry = in->entry;
            memcpy(hosts[i].ports, in->ports, 65537 * sizeof(int));
            hosts[i].addrinfo = in->addrinfo;
            hosts[i].netmask = in->netmask;
            hosts[i].extra_hosts = in->extra_hosts;
            hosts[i].num_ports = in->num_ports;
            hosts[i].next = in->next;
            in = in->next;
        }
    }

    /* build the new array */
    for(i = 0; i < num_hosts; i++)
    {
        /* get a random record */
        rand_int =
            (unsigned int) (256.0 * rand() / (RAND_MAX + 0.0)) % num_hosts;
      test:
        if(hosts[rand_int].addrinfo == 0)
        {
            rand_int++;
            rand_int %= num_hosts;
            goto test;
        }

        hosts2[i].addrinfo = hosts[rand_int].addrinfo;
        memcpy(hosts2[i].ports, hosts[rand_int].ports, 65537 * sizeof(int));
        hosts2[i].netmask = hosts[rand_int].netmask;
        hosts2[i].num_ports = hosts[rand_int].num_ports;
        hosts2[i].extra_hosts = hosts[rand_int].extra_hosts;
        hosts2[i].entry = (char *) Malloc(strlen(hosts[rand_int].entry) + 2);
        memset(hosts2[i].entry, 0, strlen(hosts[rand_int].entry) + 2);
        if(!hosts2[i].entry)
        {
            err_msg("ERROR MOVING HOSTNAME: randomizing aborted\n");
            return in;
        }
        strncpy(hosts2[i].entry, hosts[rand_int].entry,
                strlen(hosts[rand_int].entry));
        /* XXX: free(hosts[rand_int].entry); */

        /* link it */
        if(i > 0)
            hosts2[i - 1].next = &hosts2[i];

        memset(&hosts[rand_int], 0, sizeof(target_t));
        /* printf("Profile: hosts2[%d] = %s\n", i, hosts2[i].entry); printf(" 
         * hosts2[%d] = hosts[%d]\n", i, rand_int); */
    }
    // make it circular
    hosts2[num_hosts - 1].next = &hosts2[0];

    // remove the old one
    assert(hosts != NULL);
    free(hosts);
    // return the pointer
    return hosts2;
}

int
sanity_check(int value, int min, int dir)
{
    if(dir == LESS)
    {
        if(value < min)
            value = min;
        return value;
    }
    if(value > min)
        value = min;
    return value;
}

#ifndef HAVE_STRNDUP
#define HAVE_STRNDUP
static const char blah[] = "strndup_replaced!\0";
char *
strndup(const char *src, size_t n)
{
    char *dst;
    dst = malloc(n + 1);
    strncpy(dst, src, n);
    return dst;
}
#endif
